import { Setting, MarkStore, Task, MarkResult, Track } from "@/types";
import { reactive } from "vue";

const obj = {
  setting: {
    mode: "TRACK",
    examType: "SCAN_IMAGE",
    forceMode: false,
    sheetView: false,
    sheetConfig: [],
    enableAllZero: false,
    fileServer: "",
    userName: "",
    subject: <Setting["subject"]>{},
    forceSpecialTag: false,
    uiSetting: {
      "answer.paper.scale": 1,
      "score.board.collapse": false,
      "normal.mode": "keyboard",
      "paper.modal": false,
      "answer.modal": false,
      "minimap.modal": false,
      "specialTag.modal": false,
    },
    statusValue: "FORMAL",
    problemTypes: [],
    groupNumber: 0,
    groupTitle: "",
    topCount: 0,
    splitConfig: [],
    prefetchCount: 3,
    startTime: 0,
    endTime: 0,
  },
  status: <MarkStore["status"]>{},
  groups: [],
  tasks: [],
  message: null,
  currentMarkResult: undefined,
  currentTask: undefined,
  currentQuestion: undefined,
  currentScore: undefined,
  currentSpecialTag: undefined,
  markResults: [],
  historyOpen: false,
  MarkBoardTrackCollapse: false,
  historyTasks: [],
  removeScoreTracks: [],
  focusTracks: [],
  maxModalZIndex: 1020,
  minimapScrollTo: 0,
  allPaperModal: false,
  sheetViewModal: false,
  globalMask: false,
} as MarkStore;

/** 如果currentTask不存在，则返回undefined; 如果currentMarkResult不存在，则创建一个对应的markResult并返回 */
export function findCurrentTaskMarkResult() {
  if (!store.currentTask) return;
  const { libraryId } = store.currentTask;
  let markResult = store.markResults.find((m) => m.libraryId === libraryId);

  if (!markResult) {
    const { studentId } = store.currentTask;
    const statusValue = store.setting.statusValue;
    markResult = {} as MarkResult;
    markResult.libraryId = libraryId;
    markResult.studentId = studentId;
    markResult.statusValue = statusValue;
    markResult.__markStartTime = Date.now();
    markResult.__previous = store.currentTask.previous;
    markResult.trackList = store.currentTask.questionList.reduce(
      (all, c) => all.concat(c.trackList),
      [] as Array<Track>
    );
    markResult.specialTagList = [...(store.currentTask.specialTagList ?? [])];
    markResult.scoreList = store.currentTask.questionList.map((q) => q.score);
    markResult.markerScore = 0; // 后期通过 scoreList 自动更新

    store.markResults = [...store.markResults, markResult];
  }

  return store.markResults.find((m) => m.libraryId === libraryId) as MarkResult;
}

function clearMarkResultFromTask(task: Task) {
  let { libraryId } = task;
  const i = store.markResults.findIndex((s) => s.libraryId === libraryId);
  if (i >= 0) store.markResults.splice(i, 1);
}
/** 回评时，切换任务前，删除之前的回评markResult */
export function removeOldPreviousMarkResult() {
  // 清理不在当前任务中的markResult，防止回评产生的markResult造成垃圾
  store.markResults = store.markResults.filter((m) => !!!m.__previous);
}

export function removeCurrentMarkResult() {
  if (!store.currentTask) return;

  clearMarkResultFromTask(store.currentTask);

  for (const task of store.tasks) {
    // 清理不在当前任务中的markResult，防止回评产生的markResult造成垃圾
    clearMarkResultFromTask(task);
  }
}

/**
 * 返回正在评卷的状态 '' | 回评 | 打回
 */
export function getMarkStatus() {
  if (!store.currentTask) return "";

  if (store.currentTask.previous) return "回评";
  if (store.currentTask.rejected) return "打回";

  return "";
}

/**
 * 是否是扫描阅卷
 * @returns boolean
 */
export function isScanImage() {
  return store.setting.examType === "SCAN_IMAGE";
}

export const store = reactive(obj);
